%% Initiate our clean workspace and apply basic PTB default settings
sca;
close all;
clearvars;

Screen('Preference', 'SkipSyncTests', 0)
PsychDefaultSetup(2);

%% Insert DATAPixx code to open a USB connection and to initialize the controller
Datapixx('Open');
Datapixx('StopAllSchedules');
Datapixx('RegWrRd');

%% DATAPixx code to initialize the Din log
Datapixx('EnableDinDebounce');  % Filter out button bounce
Datapixx('SetDinLog');              % Configure logging with default values
Datapixx('StartDinLog');                                % Turn on logging
Datapixx('RegWrRd');

%% Configure digital input system to drive LED lights.
Datapixx('SetDinDataDirection', hex2dec('1F0000')); %Reverse direction to drive up to 5 button lights
Datapixx('RegWrRd');

% Make sure all lights are turned off before we start
Datapixx('SetDinDataOut', 0);
Datapixx('RegWrRd');


% Set LED Lights RESPONSEPixx button lights

redLight = bin2dec('10000000000000000'); % dout 16 -- notice the right-hand zeros!
yellowLight = bin2dec('100000000000000000');
greenLight = bin2dec('1000000000000000000');
blueLight = bin2dec('10000000000000000000');
whiteLight = bin2dec('100000000000000000000');

Datapixx('SetDinDataOut', redLight+greenLight);
%% % Use PTB to open up a testing window and setup the stimulus presentation code.
screenNumber = max(Screen('Screens')) % Open a widow on the display with the largest identification number.
[window, windowRect] = PsychImaging('OpenWindow', screenNumber, BlackIndex(screenNumber));
[xCenter, yCenter] = RectCenter(windowRect);
baseRect = [0 0 200 200];
centeredRect = CenterRectOnPointd(baseRect, xCenter, yCenter);

HideCursor(screenNumber);


%% Main stimulus loop 
nRep = 5;
buttonPress =zeros(1,nRep);
data = zeros(1,nRep);
tt = zeros(1,nRep);
stimulusOnsetTime = zeros(1,nRep);

for i = 1:nRep
    % Wait until all buttons are up. Use the rapid and efficient bitand
    % function to verify if all of the relevant digital input values are in
    % the correct position (1). We will see how to edit the 'up' buttons
    % later.
    
    while (bitand(Datapixx('GetDinValues'), hex2dec('FFFF')) ~= hex2dec('FFFF'))
        Datapixx('RegWrRd');
    end
    
    %'Flush' any past button presses by resseting the Din Logger.;
    Datapixx('SetDinLog');
    Datapixx('RegWrRd');
    
    Screen('FillRect', window, [1 1 1], centeredRect);
    
    %% Insert a random delay with Datapixx('GetTime');
    delay(i) =0.5 + rand/2;% 0.5s to 1 second delay
    preTrialStartTime = Datapixx('GetTime');
    trialStartTime = preTrialStartTime+ delay(i);
    
    while (Datapixx('GetTime') < trialStartTime)
        Datapixx('RegWrRd');
    end
    
    %% Insert a marker on the next frame flip with Datapixx('SetMarker');
    Datapixx('SetMarker');
    Datapixx('RegWrVideoSync');
    
    %% Display the white square with PTB's Screen('Flip')
    Screen('Flip', window);
      %% Continue looping until one of our target buttons  has been pressed. 
     
    while true % Dummy while loop. Will only exit once we detect a new logged frame and 'break' out of the loop
        Datapixx('GetDinValues')
        Datapixx('RegWrRd');
        status = Datapixx('GetDinStatus');
        if (status.newLogFrames >0) && ...
                ( ...
                (bitand(Datapixx('GetDinValues'), hex2dec('FFFF')) == hex2dec('FFFE'))...
                ||(bitand(Datapixx('GetDinValues'), hex2dec('FFFF')) == hex2dec('FFFB'))...
                )
            [data tt] = Datapixx('ReadDinLog');
            buttonPress(i) = data(end);
            timePressed(i) = tt(end);
            %% Insert DATAPixx code that reads the time value of the marker associated with the frame flip.
            stimulusOnsetTime(i) = Datapixx('GetMarker'); %
            break; %Exit out of the while loop if any button was pressed
        end
          
    end
    
    
    %% Display a black square to mask the stimulus before looping back.
    Screen('FillRect', window, [0 0 0], centeredRect);
    Screen('Flip', window);
end
%% Close our PTB stimulus window and terminate presentation
sca;
ShowCursor();

%% Calculate the reaction times and display the buttons that were pressed . 

for i = 1:nRep
    switch buttonPress(i) 
        case hex2dec('FFFE')
            ButtonID = 'red';
         case hex2dec('FFF7') 
            ButtonID = 'blue' ;
         case  hex2dec('FFFB') 
            ButtonID = 'green'; 
         case  hex2dec('FFFD') 
            ButtonID = 'yellow' ;
         case  hex2dec('FFEF') 
            ButtonID = 'white';
    end
    
    fprintf('On trial %.0f you pressed the %s button, and the reaction time was %2.2f s', i, ButtonID, timePressed(i)-stimulusOnsetTime(i));
    fprintf('\n');
end

%% Turn off LED lights.
Datapixx('SetDinDataOut', 0);
Datapixx('RegWrRd');

%% Insert DATAPixx code to close the USB connection to the data acquisition system
Datapixx('Close')

